function [Pi,c_k,s_k] = Periodogram(F,k)
%
%  Calculates the periodogram of F (a time series or a row)
%  at the point k/length(F), k \in 1..floor{length(F)/2}+1
%
%  Parameters:
%    F -- a time series (a column)
%    k -- number of frequency where to calculate periodogram
%         freq = k/length(F), k \in 1..floor{length(F)/2}+1
%
%  Comments:
%    Sum of all periodogram values is equal to the squared norm of F.
%
%  (c) Theodore Alexandrov (autossa@gmail.com)
%  Last modified: 4 May 2006 
%


N=length(F);
w=(k-1)/N; %k is an index but w starts from zero
t=(0:1:N-1)'; 

c = sum(F .* cos(2*pi*t*w));
s = sum(F .* sin(2*pi*t*w));
Pi = c^2 + s^2;

Pi=Pi*2/N;
c_k=2/N*c;
s_k=2/N*s;

if (k==1 || k==floor(N/2)+1)
	Pi = Pi/2;
	c_k = c_k/2;
	s_k = s_k/2;
end