function T = est_period(Uharm)

% 
%  Estimate period of a harmonic reconstructed by ET1, ET2
% 
%  Uharm = [U1 U2] -- matrix of "harmonic" eigen vectors 
%
%   Theodore Alexandrov (autossa@gmail.com)
%   Last modified: Jun 22, 2006

% calculate verticality coefficient
lastvals=Uharm(end,:);
v2 = lastvals*lastvals';

% calculate LRF coefficients
LRF_coefs = 1/(1-v2) * Uharm(1:end-1,:)*lastvals';

% char.polynomial coefs
chpol_coefs = [1 -LRF_coefs(2:end)'];

% char.polynomial roots
chpol_roots = roots(chpol_coefs);

% find a root with maximal module
maxroot=chpol_roots(1);
maxmod=0;
for k=1:length(chpol_roots) 
    root=chpol_roots(k);
    rtmod = sqrt(root*conj(root));
    if imag(root)>1e-5 && rtmod > maxmod
       maxroot = root;
       maxmod = rtmod;
    end
end

T=2*pi/acos( real(maxroot)/maxmod );
