function varargout = gautotrend(varargin)
% GAUTOTREND M-file for gautotrend.fig

%   Extracts trend of given time series
%
%   Usage:
%	gautotrend(ts, L, omega, maxET, C0min, C0max, C0step, RDeltaThreshold, C0eps)
%	gautotrend(ts, L, omega0)
%
%
%   Mandatory parameters: 
%	ts -- a column, containing time series values
%	L -- window width (1<L<time series length), 
%	     usually L=floor(N/2)
%	omega0 -- low freq. border, low freq. interval = [0, omega0],
%	          0<omega0<0.5, for monthly data omega0=0.08
%
%   Optional parameters:
%	maxET -- amount of first ETs to be examined, usually floor(L/2)
%	C0min -- where search of the best C0 starts, usually 0.5
%	C0max -- where it finishes, usually 1
%	C0step -- step of the search, usually 0.1
%	RDeltaThreshold -- a threshold for R measure deltas, C0max = arg min_{C0} { R(C0+C0step)-R(C0) },
%	                   usually 0.05
%	C0eps -- optimal C0=C0max-C0eps, usually 0
%
%
%   Theodore Alexandrov (autossa@gmail.com)
%
% Last Modified by GUIDE v2.5 28-Jun-2006 18:48:58

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @gautotrend_OpeningFcn, ...
                   'gui_OutputFcn',  @gautotrend_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before gautotrend is made visible.
function gautotrend_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to gautotrend (see VARARGIN)

% Choose default command line output for gautotrend
handles.output = hObject;


if length(varargin)<3
    uiwait( errordlg('You must specify time series, L and omega0', 'Error') );
end

% mandatory parameters
if length(varargin)>=1
   handles.givendata.ts = varargin{1};
   handles.data.ts = handles.givendata.ts;
end
if length(varargin)>=2
   handles.givendata.L = varargin{2};
   handles.data.L = handles.givendata.L;
end
if length(varargin)>=3  
   handles.givendata.omega0 = varargin{3};
   handles.data.omega0 = handles.givendata.omega0;
end

% optional parameters
if length(varargin)>=4  
   handles.givendata.maxET = varargin{4};
   handles.data.maxET = handles.givendata.maxET;
else
   handles.data.maxET = floor(handles.data.L/2);
end
if length(varargin)>=5
   handles.givendata.C0left = varargin{5};
   handles.data.C0left = handles.givendata.C0left;
else
   handles.data.C0left = 0.5; %default value
end
if length(varargin)>=6
   handles.givendata.C0right = varargin{6};
   handles.data.C0right = handles.givendata.C0right; 
else
   handles.data.C0right = 1; % default value
end
if length(varargin)>=7
   handles.givendata.C0step = varargin{7};
   handles.data.C0step = handles.givendata.C0step;
else
   handles.data.C0step = 0.01; % default value
end
if length(varargin)>=8
   handles.givendata.RDeltaThresh = varargin{8};
   handles.data.RDeltaThresh = handles.givendata.RDeltaThresh;
else
   handles.data.RDeltaThresh = 0.05; % default value
end
if length(varargin)>=9
   handles.givendata.C0eps = varargin{9};
   handles.data.C0eps = handles.givendata.C0eps;
else
   handles.data.C0eps = 0; % default value
end

handles.data.MSEthreshFORperresid = 0.05;

handles.settings.centred_btnstr='Original';
handles.settings.notcentred_btnstr='Center t.s.';
handles.settings.resuts_filename = 'autotrend.res';

% clear all graphs, plot ts and clear all made changes in settings
reset_all(hObject, handles);

% Update handles structure
guidata(hObject, handles);



% --- clear all graphs, plot ts and clear all made changes in settings
function reset_all(hObject, handles);
set(handles.center_btn, 'String', handles.settings.notcentred_btnstr);
handles.operations.centred=0;

set(handles.L_input, 'String', num2str(handles.data.L));
set(handles.omega0_input, 'String', num2str(handles.data.omega0));
set(handles.maxET_input, 'String', num2str(handles.data.maxET));
set(handles.C0left_input, 'String', num2str(handles.data.C0left));
set(handles.C0right_input, 'String', num2str(handles.data.C0right));
set(handles.C0step_input, 'String', num2str(handles.data.C0step));
set(handles.RDeltaThresh_input, 'String', num2str(handles.data.RDeltaThresh));
set(handles.C0eps_input, 'String', num2str(handles.data.C0eps));

clear_all_graphs(handles);
plot_ts(handles);


% Update handles structure
guidata(hObject, handles);




% --- clear all graphs
function clear_all_graphs(handles);

axes(handles.residual_axes);
cla;
reset(gca);
box on;
axes(handles.Rdeltas_axes);
cla;
reset(gca);
box on;
axes(handles.Ccrit_axes);
cla;
reset(gca);
box on;



% --- plot handles.data.ts
function plot_ts(handles);

% plot time series 
axes(handles.trend_axes)
plot(handles.data.ts);
if isfield(handles, 'operations') && isfield(handles.operations, 'centred') && ...
   handles.operations.centred==1
   legend('Centred');
else
   legend('Original');
end
xlim([1 length(handles.data.ts)])
grid on;
set(gca, 'XMinorGrid', 'on');





% --- Outputs from this function are returned to the command line.
function varargout = gautotrend_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;



function L_input_Callback(hObject, eventdata, handles)
% hObject    handle to L_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of L_input as text
%        str2double(get(hObject,'String')) returns contents of L_input as a double

handles.data.L = str2num(get(hObject, 'String'));

if handles.data.maxET > handles.data.L
    errordlg(['Maximal ET (', num2str(handles.data.maxET), ') < L (', num2str(handles.data.L) ,'). Input correct maximal ET number!']);
end


% Update handles structure
guidata(hObject, handles);




% --- Executes during object creation, after setting all properties.
function L_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to L_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end





function C0left_input_Callback(hObject, eventdata, handles)
% hObject    handle to C0left_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of C0left_input as text
%        str2double(get(hObject,'String')) returns contents of C0left_input as a double
handles.data.C0left = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function C0left_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to C0left_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function C0right_input_Callback(hObject, eventdata, handles)
% hObject    handle to C0right_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of C0right_input as text
%        str2double(get(hObject,'String')) returns contents of C0right_input as a double

handles.data.C0right = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);



% --- Executes during object creation, after setting all properties.
function C0right_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to C0right_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function C0step_input_Callback(hObject, eventdata, handles)
% hObject    handle to C0step_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of C0step_input as text
%        str2double(get(hObject,'String')) returns contents of C0step_input as a double

handles.data.C0step = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);



% --- Executes during object creation, after setting all properties.
function C0step_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to C0step_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function omega0_input_Callback(hObject, eventdata, handles)
% hObject    handle to omega0_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of omega0_input as text
%        str2double(get(hObject,'String')) returns contents of omega0_input as a double

handles.data.omega0 = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function omega0_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to omega0_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function RDeltaThresh_input_Callback(hObject, eventdata, handles)
% hObject    handle to RDeltaThresh_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of RDeltaThresh_input as text
%        str2double(get(hObject,'String')) returns contents of RDeltaThresh_input as a double

handles.data.RDeltaThresh = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function RDeltaThresh_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to RDeltaThresh_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function C0eps_input_Callback(hObject, eventdata, handles)
% hObject    handle to C0eps_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of C0eps_input as text
%        str2double(get(hObject,'String')) returns contents of C0eps_input as a double

handles.data.C0eps = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);



% --- Executes during object creation, after setting all properties.
function C0eps_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to C0eps_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in periodicityOFresidual_btn.
function periodicityOFresidual_btn_Callback(hObject, eventdata, handles)
% hObject    handle to periodicityOFresidual_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if isfield(handles, 'results') && isfield(handles.results, 'tr')
    gautoperiodicity(handles.data.ts - handles.results.tr, handles.data.L, handles.data.MSEthreshFORperresid);
else
    errordlg('No results. Extract trend first.', 'Error');
end



function MSEthreshFORperresid_input_Callback(hObject, eventdata, handles)
% hObject    handle to MSEthreshFORperresid_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of MSEthreshFORperresid_input as text
%        str2double(get(hObject,'String')) returns contents of MSEthreshFORperresid_input as a double

handles.data.MSEthreshFORperresid = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function MSEthreshFORperresid_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to MSEthreshFORperresid_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end




% --- Executes on button press in periodicityOFts_btn.
function periodicityOFts_btn_Callback(hObject, eventdata, handles)
% hObject    handle to periodicityOFts_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

gautoperiodicity(handles.data.ts, handles.data.L, handles.data.MSEthreshFORperresid);




function maxET_input_Callback(hObject, eventdata, handles)
% hObject    handle to maxET_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of maxET_input as text
%        str2double(get(hObject,'String')) returns contents of maxET_input as a double

handles.data.maxET = str2num(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);




% --- Executes during object creation, after setting all properties.
function maxET_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to maxET_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end





% --- Executes on button press in center_btn.
function center_btn_Callback(hObject, eventdata, handles)
% hObject    handle to center_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

centred = 0;
if isfield(handles, 'operations') && isfield(handles.operations, 'centred')
        centred = handles.operations.centred;
end

if ~centred
    handles.data.ts = handles.data.ts-sum(handles.data.ts)/length(handles.data.ts);
    set(handles.center_btn, 'String', handles.settings.centred_btnstr);
    
    handles.operations.centred=1;
end

if centred
    handles.data.ts = handles.givendata.ts;
    set(handles.center_btn, 'String', handles.settings.notcentred_btnstr);
    
    handles.operations.centred=0;
end

clear_all_graphs(handles);
plot_ts(handles);


% Update handles structure
guidata(hObject, handles);


% --- Executes on button press in scrshot_all_btn.
function scrshot_all_btn_Callback(hObject, eventdata, handles)
% hObject    handle to scrshot_all_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if ~isfield(handles, 'operations') || ~isfield(handles.operations, 'screenshots')
    handles.operations.screenshots=1;
else
    handles.operations.screenshots=handles.operations.screenshots+1;
end


frame=getframe(gcf);
img=frame2im(frame);
scrfilename=['screenshot', num2str(handles.operations.screenshots), '.bmp'];
imwrite(img,scrfilename, 'bmp')


helpdlg(['Screenshot was saved to "', scrfilename, '"']);

% Update handles structure
guidata(hObject, handles);




% --- Executes on button press in scrshot_trresid_btn.
function scrshot_trresid_btn_Callback(hObject, eventdata, handles)
% hObject    handle to scrshot_trresid_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if ~isfield(handles, 'operations') || ~isfield(handles.operations, 'screenshots')
    handles.operations.screenshots=1;
else
    handles.operations.screenshots=handles.operations.screenshots+1;
end

y0=230;
x0=1;
height=700;
width=980;
frame=getframe(gcf, [x0,y0,width-x0,height-y0]);
img=frame2im(frame);
scrfilename=['screenshot', num2str(handles.operations.screenshots), '.bmp'];
imwrite(img,scrfilename, 'bmp')

helpdlg(['Screenshot was saved to "', scrfilename, '"']);

% Update handles structure
guidata(hObject, handles);



% --- Executes on button press in saveresults_btn.
function saveresults_btn_Callback(hObject, eventdata, handles)
% hObject    handle to saveresults_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if ~isfield(handles, 'results')
    errordlg('No results', 'Error');
    return;
end
    
save_autotrend(handles.settings.resuts_filename, ...
               handles.data.ts, ...
               handles.data.L, ...
               handles.data.omega0, ...
               handles.data.maxET, ...
               handles.data.C0left, ...
               handles.data.C0right, ...
               handles.data.C0step, ...
               handles.data.RDeltaThresh, ...
               handles.data.C0eps, ...
               ...
               handles.results.tr, ...
               handles.results.trETs, ...
               handles.results.C0max, ...
               handles.results.C0maxDelta, ...
               handles.results.Cvals_eachET, ...
               handles.results.C0s, ...
               handles.results.CDeltas);

helpdlg(['The results were saved to "', handles.settings.resuts_filename, '"']);




% --- Executes on button press in recalculate.
function recalculate_Callback(hObject, eventdata, handles)
% hObject    handle to recalculate (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% get the structure in the subfunction

if handles.data.maxET > handles.data.L
    errordlg(['Maximal ET (', num2str(handles.data.maxET), ') < L (', num2str(handles.data.L) ,'). Input correct maximal ET number!']);
    return;
end

ts = handles.data.ts;
L = handles.data.L;
omega0 = handles.data.omega0;

if ~exist('ts', 'var') || ~exist('L', 'var') || ~exist('omega0', 'var')
    uiwait( errordlg('Time series or L or omega0 wasn''t specified', 'Error', 'on') );
    return;
end

maxET = handles.data.maxET;
C0left = handles.data.C0left;
C0right = handles.data.C0right;
C0step = handles.data.C0step;
RDeltaThresh = handles.data.RDeltaThresh ;
C0eps = handles.data.C0eps;


progressbar = waitbar(0,'Please wait...');

[tr,trETs,C0max,C0maxDelta,Cvals_eachET,C0s,CDeltas] = ...
        autotrend(ts, L, omega0, ...
                  maxET, C0left, C0right, C0step, RDeltaThresh, C0eps, ...
                  progressbar);
              
close(progressbar);

C0 = C0max - C0eps;

handles.results.tr = tr;
handles.results.trETs = trETs;
handles.results.C0max = C0max;
handles.results.C0 = C0;
handles.results.C0maxDelta = C0maxDelta;
handles.results.Cvals_eachET = Cvals_eachET;
handles.results.C0s = C0s;
handles.results.CDeltas = CDeltas;

% plot time series and trend 
axes(handles.trend_axes)
plot(ts);
hold on;
plot(tr, 'r');
hold off;

if isfield(handles, 'operations') && isfield(handles.operations, 'centred') && ...
   handles.operations.centred==1
    legts = 'Centred';
else
    legts = 'Original';
end
legend(legts, ['Trend, L=', num2str(L), ', \omega_0=', num2str(omega0), ', C_0=', num2str(C0), ...
       ', ETs=', ETsnum2str(trETs)]);
xlim([1 length(ts)])
grid on;
set(gca, 'XMinorGrid', 'on');

% plot residual time series (original - trend)
axes(handles.residual_axes)
plot(ts-tr);
legend('Residual');
xlim([1 length(ts)])
grid on;
set(gca, 'XMinorGrid', 'on');

% plot R-measure deltas
axes(handles.Rdeltas_axes)
plot(C0s, CDeltas, 'b.-');
hold on;
plot(C0s,ones(size(CDeltas))*RDeltaThresh,'r');
hold off;
legend(['\Delta R-measure, Cmax: \Delta(', num2str(C0max), ')=', num2str(RDeltaThresh)], ...
       ['R-measure delta threshold=', num2str(RDeltaThresh)], ...
       'Location', 'NorthWest');
ylim([min(CDeltas) max(CDeltas)]);
xlim([C0s(1) C0s(end)]);


%plot C-criterion values
axes(handles.Ccrit_axes)
bar(Cvals_eachET);
hold on;
stairs(ones(size(Cvals_eachET))*C0, 'r');
hold off;
legend('Values of C for each ET', ['C_0=Cmax-C_\epsilon=', num2str(C0)]);
xlim([1 length(Cvals_eachET)]);
ylim([0 1.2]);
set(gca, 'XMinorGrid', 'on');

% Update handles structure
guidata(hObject, handles);






