function [trend, trend_ETs] = trend(ts, L, omega0, C0, maxET)
%
%  Extracts a trend from a given time series
%
%  Parameters:
%    ts -- a time series (a column)
%    L -- window length (1<L<N, N is time series length)
%    omega0 -- an upper boundary of low frequencies interval [0, omega0], 
%              0<omeg0<0.5, for monthly data omega0=0.08
%    C0 -- a threshold value for the LowFreq method of trend identification
%
%  Returns:
%    trend -- a trend time series (a column)
%    trend_ETs -- a list with trend eigentriples (starting with 1)
%
%  (c) Theodore Alexandrov (autossa@gmail.com)
%  Last modified: 6 Oct 2006 
%

if ~exist('maxET','var')
   maxET=L;
end


% SSA
[sing_values,U,V] = ssa(ts, L);

d=length(sing_values);

if length(sing_values)>maxET
    sing_values(maxET+1,end)=0;
    U(:,maxET+1:end)=0;
    V(:,maxET+1:end)=0;
else
    d=min(d, maxET);
end

% identification
trend_ETs = LF_identify(U, omega0, C0, d);

% reconstruction
trend = ssa_reconstruct(sing_values, U, V, trend_ETs);

